//+------------------------------------------------------------------+
//|                                                     TickSave.mq4 |
//|                                 Copyright  2006-2015, komposter |
//|                                          http://www.komposter.me |
//+------------------------------------------------------------------+
#property copyright		"Copyright  2006-2015, komposter"
#property link				"http://www.komposter.me"
#property version	 		"2.0"
#property strict

input string	SymbolsList		= "EURUSD,GBPUSD,AUDUSD,USDCAD,USDJPY,USDCHF";

string SymbolsArray[32];
double preBid[32];
int FileHandle[32], SymbolsCount = 0;

string	ServerName		= "";
int		CurYear			= 0;
int		CurMonth			= 0;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int OnInit()
{
	ServerName		= AccountServer();
	CurYear			= Year();
	CurMonth			= Month();

	//    SymbolsList  
	if ( !PrepareSymbolsList() ) { return(INIT_FAILED); }

	//  
	if ( !OpenFiles() ) { return(INIT_FAILED); }

	//  
	if ( !EventSetMillisecondTimer( 100 ) )
	{
		Alert( "Can't set the timer!" );
		return(INIT_FAILED);
	}

	return(INIT_SUCCEEDED);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
{
	//  
	CloseFiles();

	Comment("");
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void OnTimer()
{
	//   ,    -       
	//   ,    -       
	if ( ServerName != AccountServer() || CurYear != Year() || CurMonth != Month() )
	{
		CloseFiles();
		if ( PrepareSymbolsList() && OpenFiles() )
		{
			ServerName		= AccountServer();
			CurYear			= Year();
			CurMonth			= Month();
		}
	}

	//   
	WriteTick();
}

//+------------------------------------------------------------------+
//|    SymbolsList  
//+------------------------------------------------------------------+
bool PrepareSymbolsList()
{
	int		curchar = 0;
	int		len = StringLen( SymbolsList ), curSymbol;
	string	cur_symbol = ""; SymbolsCount = 0;

	//----     
	ArrayResize( SymbolsArray, 32 );

	//----      
	for ( int pos = 0; pos <= len; pos ++ )
	{
		curchar = StringGetChar( SymbolsList, pos );
		//----    -        ,
		if ( curchar != ',' && pos != len )
		{
			//----  -     
			cur_symbol = cur_symbol + CharToStr( uchar(curchar) );
			continue;
		}
		//----    ,   -    ,
		else
		{ 
			//---- ,   cur_symbol -   .  :
			MarketInfo( cur_symbol, MODE_BID );
			if ( cur_symbol == "" || GetLastError() == 4106 )
			{
				Alert( "  \"", cur_symbol, "\"!!!" );
				return(false);
			}

			//----    , ,       :
			bool Uniq = true;
			for ( curSymbol = 0; curSymbol < SymbolsCount; curSymbol ++ )
			{
				if ( cur_symbol == SymbolsArray[curSymbol] )
				{
					Uniq = false;
					break;
				}
			}

			//----       ,  ,    :
			if ( Uniq )
			{
				SymbolsArray[SymbolsCount] = cur_symbol;
				SymbolsCount ++;
				if ( SymbolsCount > 31 )
				{
					Alert( "  !    32 !" );
					return(false);
				}
			}

			//----   
			cur_symbol = "";
		}
	}

	//----       , 
	if ( SymbolsCount <= 0 )
	{
		Alert( "    !!!" );
		return(false);
	}
	
	//----      - :
	ArrayResize		( SymbolsArray	, SymbolsCount );
	ArrayResize		( preBid			, SymbolsCount );
	ArrayInitialize( preBid			, -1 				);
	ArrayResize		( FileHandle	, SymbolsCount );
	ArrayInitialize( FileHandle	, -1 				);

	//----  :
	string uniq_symbols_list = SymbolsArray[0];
	for ( curSymbol = 1; curSymbol < SymbolsCount; curSymbol ++ )
	{
		if ( curSymbol == SymbolsCount - 1 )
		{ uniq_symbols_list = uniq_symbols_list + "  " + SymbolsArray[curSymbol]; }
		else
		{ uniq_symbols_list = uniq_symbols_list + ", " + SymbolsArray[curSymbol]; }
	}
	Comment( AccountServer(), ":  ", SymbolsCount, " (-,-):\n", uniq_symbols_list );

	return(true);
}

//+------------------------------------------------------------------+
//|  ,     
//+------------------------------------------------------------------+
bool OpenFiles()
{
	int _GetLastError;
	for ( int curSymbol = 0; curSymbol < SymbolsCount; curSymbol ++ )
	{
		string FileName = StringConcatenate( "[Ticks]\\", AccountServer(), "\\", SymbolsArray[curSymbol], "_", Year(), ".", strMonth(), ".csv" );
		FileHandle[curSymbol] = FileOpen( FileName, FILE_READ | FILE_WRITE | FILE_CSV | FILE_ANSI | FILE_SHARE_READ );

		if ( FileHandle[curSymbol] < 0 )
		{
			_GetLastError = GetLastError();
			Alert( "FileOpen( " + FileName + ", FILE_READ | FILE_WRITE | FILE_CSV | FILE_ANSI | FILE_SHARE_READ ) - Error #", _GetLastError );
			return(false);
		}

		if ( !FileSeek( FileHandle[curSymbol], 0, SEEK_END ) )
		{
			_GetLastError = GetLastError();
			Alert( "FileSeek( ", FileHandle[curSymbol], ", 0, SEEK_END ) - Error #", _GetLastError );
			return(false);
		}

		preBid[curSymbol] = MarketInfo( SymbolsArray[curSymbol], MODE_BID );
	}
	return(true);
}

//+------------------------------------------------------------------+
//|   
//+------------------------------------------------------------------+
void WriteTick()
{
	int _GetLastError; double curBid, curPoint; int curDigits;
	for ( int curSymbol = 0; curSymbol < SymbolsCount; curSymbol ++ )
	{
		if ( FileHandle[curSymbol] < 0 ) { continue; }

		curBid = MarketInfo( SymbolsArray[curSymbol], MODE_BID );
		curDigits = (int)MarketInfo( SymbolsArray[curSymbol], MODE_DIGITS );
		curPoint = MarketInfo( SymbolsArray[curSymbol], MODE_POINT );

		if ( MathAbs( curBid - preBid[curSymbol] ) < curPoint/2.0 ) { continue; }

		//if ( !FileSeek( FileHandle[curSymbol], 0, SEEK_END ) )
		//{
		//	_GetLastError = GetLastError();
		//	Alert( "FileSeek( ", FileHandle[curSymbol], ", 0, SEEK_END ) - Error #", _GetLastError );
		//	continue;
		//}

		if ( FileWrite( FileHandle[curSymbol], TimeToStr( TimeCurrent(), TIME_DATE | TIME_SECONDS ), DoubleToStr( curBid, curDigits ) ) <= 0 )
		{
			_GetLastError = GetLastError();
			Alert( "FileWrite() Error #", _GetLastError );
		}

		preBid[curSymbol] = curBid;

		FileFlush( FileHandle[curSymbol] );
	}
}

//+------------------------------------------------------------------+
//|   
//+------------------------------------------------------------------+
void CloseFiles()
{
	for ( int curSymbol = 0; curSymbol < SymbolsCount; curSymbol ++ )
	{
		if ( FileHandle[curSymbol] > 0 )
		{
			FileClose( FileHandle[curSymbol] );
			FileHandle[curSymbol] = -1;
		}
	}
}

string strMonth()
{
	if ( Month() < 10 ) return( StringConcatenate( "0", Month() ) );
	return(string(Month()));
}